@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -29,7 +29,7 @@ import os
 import time
 
 from .authentication import get_auth_plugin
-from .catch23 import PY2, isstr
+from .catch23 import PY2, isstr, UNICODE_TYPES
 from .constants import (
     ClientFlag, ServerCmd, ServerFlag,
     flag_is_set, ShutdownType, NET_BUFFER_LENGTH
@@ -90,8 +90,14 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._auth_plugin = None
         self._pool_config_version = None
 
-        if len(kwargs) > 0:
-            self.connect(**kwargs)
+        if kwargs:
+            try:
+                self.connect(**kwargs)
+            except:
+                # Tidy-up underlying socket on failure
+                self.close()
+                self._socket = None
+                raise
 
     def _do_handshake(self):
         """Get the handshake from the MySQL server"""
@@ -109,6 +115,14 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._server_version = self._check_server_version(
             handshake['server_version_original'])
 
+        if not handshake['capabilities'] & ClientFlag.SSL:
+            self._client_flags &= ~ClientFlag.SSL
+            if self._ssl.get('verify_cert'):
+                raise errors.InterfaceError("SSL is required but the server "
+                                            "doesn't support it", errno=2026)
+        elif not self._ssl_disabled:
+            self._client_flags |= ClientFlag.SSL
+
         if handshake['capabilities'] & ClientFlag.PLUGIN_AUTH:
             self.set_client_flags([ClientFlag.PLUGIN_AUTH])
 
@@ -126,11 +140,16 @@ class MySQLConnection(MySQLConnectionAbstract):
         reply back. Raises any error coming from MySQL.
         """
         self._ssl_active = False
-        if client_flags & ClientFlag.SSL and ssl_options:
+        if client_flags & ClientFlag.SSL:
             packet = self._protocol.make_auth_ssl(charset=charset,
                                                   client_flags=client_flags)
             self._socket.send(packet)
-            self._socket.switch_to_ssl(**ssl_options)
+            self._socket.switch_to_ssl(ssl_options.get('ca'),
+                                       ssl_options.get('cert'),
+                                       ssl_options.get('key'),
+                                       ssl_options.get('verify_cert') or False,
+                                       ssl_options.get('cipher'),
+                                       ssl_options.get('version', None))
             self._ssl_active = True
 
         packet = self._protocol.make_auth(
@@ -153,6 +172,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         Raises NotSupportedError when we get the old, insecure password
         reply back. Raises any error coming from MySQL.
         """
+        auth = None
+        new_auth_plugin = self._auth_plugin or self._handshake["auth_plugin"]
         packet = self._socket.recv()
         if packet[4] == 254 and len(packet) == 5:
             raise errors.NotSupportedError(
@@ -168,12 +189,22 @@ class MySQLConnection(MySQLConnectionAbstract):
             response = auth.auth_response()
             self._socket.send(response)
             packet = self._socket.recv()
-            if packet[4] != 1:
-                return self._handle_ok(packet)
-            else:
-                auth_data = self._protocol.parse_auth_more_data(packet)
+
+        if packet[4] == 1:
+            auth_data = self._protocol.parse_auth_more_data(packet)
+            auth = get_auth_plugin(new_auth_plugin)(
+                auth_data, password=password, ssl_enabled=self._ssl_active)
+            if new_auth_plugin == "caching_sha2_password":
+                response = auth.auth_response()
+                if response:
+                    self._socket.send(response)
+                packet = self._socket.recv()
+
+        if packet[4] == 0:
+            return self._handle_ok(packet)
         elif packet[4] == 255:
             raise errors.get_exception(packet)
+        return None
 
     def _get_connection(self, prtcls=None):
         """Get connection based on configuration
@@ -183,7 +214,6 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns subclass of MySQLBaseSocket.
         """
-        # pylint: disable=R0204
         conn = None
         if self.unix_socket and os.name != 'nt':
             conn = MySQLUnixSocket(unix_socket=self.unix_socket)
@@ -191,7 +221,7 @@ class MySQLConnection(MySQLConnectionAbstract):
             conn = MySQLTCPSocket(host=self.server_host,
                                   port=self.server_port,
                                   force_ipv6=self._force_ipv6)
-        # pylint: enable=R0204
+
         conn.set_connection_timeout(self._connection_timeout)
         return conn
 
@@ -206,9 +236,11 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._socket = self._get_connection()
         self._socket.open_connection()
         self._do_handshake()
+
         self._do_auth(self._user, self._password,
                       self._database, self._client_flags, self._charset_id,
                       self._ssl)
+
         self.set_converter_class(self._converter_class)
         if self._client_flags & ClientFlag.COMPRESS:
             self._socket.recv = self._socket.recv_compressed
@@ -232,9 +264,10 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         try:
             self.cmd_quit()
-            self._socket.close_connection()
         except (AttributeError, errors.Error):
             pass  # Getting an exception would mean we are disconnected.
+        self._socket.close_connection()
+
     disconnect = close
 
     def _send_cmd(self, command, argument=None, packet_number=0, packet=None,
@@ -420,7 +453,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         Returns a tuple.
         """
         (rows, eof) = self.get_rows(count=1, binary=binary, columns=columns)
-        if len(rows):
+        if rows:
             return (rows[0], eof)
         return (None, eof)
 
@@ -438,8 +471,11 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         try:
             if binary:
+                charset = self.charset
+                if charset == 'utf8mb4':
+                    charset = 'utf8'
                 rows = self._protocol.read_binary_result(
-                    self._socket, columns, count)
+                    self._socket, columns, count, charset)
             else:
                 rows = self._protocol.read_text_result(self._socket,
                                                        self._server_version,
@@ -517,10 +553,9 @@ class MySQLConnection(MySQLConnectionAbstract):
         Returns a generator.
         """
         if not isinstance(statements, bytearray):
-            if isstr(statements):
-                statements = bytearray(statements.encode('utf-8'))
-            else:
-                statements = bytearray(statements)
+            if isstr(statements) and isinstance(statements, UNICODE_TYPES):
+                statements = statements.encode('utf8')
+            statements = bytearray(statements)
 
         # Handle the first query result
         yield self._handle_result(self._send_cmd(ServerCmd.QUERY, statements))
@@ -933,6 +968,17 @@ class MySQLConnection(MySQLConnectionAbstract):
         eof = self._handle_eof(self._socket.recv())
         return (column_count, columns, eof)
 
+    def cmd_stmt_fetch(self, statement_id, rows=1):
+        """Fetch a MySQL statement Result Set
+
+        This method will send the FETCH command to MySQL together with the
+        given statement id and the number of rows to fetch.
+        """
+        packet = self._protocol.make_stmt_fetch(statement_id, rows)
+        self.unread_result = False
+        self._send_cmd(ServerCmd.STMT_FETCH, packet, expect_response=False)
+        self.unread_result = True
+
     def cmd_stmt_prepare(self, statement):
         """Prepare a MySQL statement
 
