@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -29,8 +29,9 @@ import re
 import weakref
 
 from . import errors
-from .abstracts import MySQLCursorAbstract
+from .abstracts import MySQLCursorAbstract, NAMED_TUPLE_CACHE
 from .catch23 import PY2
+from .constants import ServerFlag
 
 SQL_COMMENT = r"\/\*.*?\*\/"
 RE_SQL_COMMENT = re.compile(
@@ -60,6 +61,7 @@ RE_SQL_FIND_PARAM = re.compile(
 
 ERR_NO_RESULT_TO_FETCH = "No result set to fetch from"
 
+MAX_RESULTS = 4294967295
 
 class _ParamSubstitutor(object):
     """
@@ -186,7 +188,7 @@ class CursorBase(MySQLCursorAbstract):
         """
         pass
 
-    def executemany(self, operation, seqparams):
+    def executemany(self, operation, seq_params):
         """Execute the given operation multiple times
 
         The executemany() method will execute the operation iterating
@@ -491,7 +493,7 @@ class MySQLCursor(CursorBase):
 
         i = 0
         while True:
-            result = next(query_iter)
+            result = next(query_iter)  # pylint: disable=R1708
             self._reset_result()
             self._handle_result(result)
             try:
@@ -554,15 +556,15 @@ class MySQLCursor(CursorBase):
         if multi:
             self._executed_list = []
             return self._execute_iter(self._connection.cmd_query_iter(stmt))
-        else:
-            try:
-                self._handle_result(self._connection.cmd_query(stmt))
-            except errors.InterfaceError:
-                if self._connection._have_next_result:  # pylint: disable=W0212
-                    raise errors.InterfaceError(
-                        "Use multi=True when executing multiple statements")
-                raise
-            return None
+
+        try:
+            self._handle_result(self._connection.cmd_query(stmt))
+        except errors.InterfaceError:
+            if self._connection._have_next_result:  # pylint: disable=W0212
+                raise errors.InterfaceError(
+                    "Use multi=True when executing multiple statements")
+            raise
+        return None
 
     def _batch_insert(self, operation, seq_params):
         """Implements multi row insert"""
@@ -575,8 +577,7 @@ class MySQLCursor(CursorBase):
             """
             if match.group(1):
                 return ""
-            else:
-                return match.group(2)
+            return match.group(2)
 
         tmp = re.sub(RE_SQL_ON_DUPLICATE, '',
                      re.sub(RE_SQL_COMMENT, remove_comments, operation))
@@ -587,11 +588,11 @@ class MySQLCursor(CursorBase):
                 "Failed rewriting statement for multi-row INSERT. "
                 "Check SQL syntax."
             )
-        fmt = matches.group(1).encode(self._connection.charset)
+        fmt = matches.group(1).encode(self._connection.python_charset)
         values = []
 
         try:
-            stmt = operation.encode(self._connection.charset)
+            stmt = operation.encode(self._connection.python_charset)
             for params in seq_params:
                 tmp = fmt
                 if isinstance(params, dict):
@@ -610,8 +611,7 @@ class MySQLCursor(CursorBase):
                 stmt = stmt.replace(fmt, b','.join(values), 1)
                 self._executed = stmt
                 return stmt
-            else:
-                return None
+            return None
         except (UnicodeDecodeError, UnicodeEncodeError) as err:
             raise errors.ProgrammingError(str(err))
         except errors.Error:
@@ -656,7 +656,7 @@ class MySQLCursor(CursorBase):
         if re.match(RE_SQL_INSERT_STMT, operation):
             if not seq_params:
                 self._rowcount = 0
-                return
+                return None
             stmt = self._batch_insert(operation, seq_params)
             if stmt is not None:
                 return self.execute(stmt)
@@ -675,6 +675,7 @@ class MySQLCursor(CursorBase):
             # Raise whatever execute() raises
             raise
         self._rowcount = rowcnt
+        return None
 
     def stored_results(self):
         """Returns an iterator for stored results
@@ -756,12 +757,10 @@ class MySQLCursor(CursorBase):
             can_consume_results = self._connection._consume_results
             for result in self._connection.cmd_query_iter(call):
                 self._connection._consume_results = False
-                # pylint: disable=R0204
                 if self._raw:
                     tmp = MySQLCursorBufferedRaw(self._connection._get_self())
                 else:
                     tmp = MySQLCursorBuffered(self._connection._get_self())
-                # pylint: enable=R0204
                 tmp._executed = "(a result of {0})".format(call)
                 tmp._handle_result(result)
                 if tmp._warnings is not None:
@@ -776,9 +775,9 @@ class MySQLCursor(CursorBase):
                 self.execute(select)
                 self._stored_results = results
                 return self.fetchone()
-            else:
-                self._stored_results = results
-                return ()
+
+            self._stored_results = results
+            return ()
 
         except errors.Error:
             raise
@@ -813,7 +812,7 @@ class MySQLCursor(CursorBase):
             raise errors.InterfaceError(
                 "Failed getting warnings; %s" % err)
 
-        if len(res):
+        if res:
             return res
 
         return None
@@ -1080,8 +1079,38 @@ class MySQLCursorPrepared(MySQLCursor):
         self._prepared = None
         self._binary = True
         self._have_result = None
+        self._last_row_sent = False
+        self._cursor_exists = False
 
-    def callproc(self, *args, **kwargs):
+    def reset(self, free=True):
+        if self._prepared:
+            try:
+                self._connection.cmd_stmt_close(self._prepared['statement_id'])
+            except errors.Error:
+                # We tried to deallocate, but it's OK when we fail.
+                pass
+            self._prepared = None
+        self._last_row_sent = False
+        self._cursor_exists = False
+
+    def _handle_noresultset(self, res):
+        self._handle_server_status(res.get('status_flag',
+                                           res.get('server_status', 0)))
+        super(MySQLCursorPrepared, self)._handle_noresultset(res)
+
+    def _handle_server_status(self, flags):
+        """Check for SERVER_STATUS_CURSOR_EXISTS and
+           SERVER_STATUS_LAST_ROW_SENT flags set by the server.
+        """
+        self._cursor_exists = flags & ServerFlag.STATUS_CURSOR_EXISTS != 0
+        self._last_row_sent = flags & ServerFlag.STATUS_LAST_ROW_SENT != 0
+
+    def _handle_eof(self, eof):
+        self._handle_server_status(eof.get('status_flag',
+                                           eof.get('server_status', 0)))
+        super(MySQLCursorPrepared, self)._handle_eof(eof)
+
+    def callproc(self, procname, args=()):
         """Calls a stored procedue
 
         Not supported with MySQLCursorPrepared.
@@ -1094,13 +1123,7 @@ class MySQLCursorPrepared(MySQLCursor):
         This method will try to deallocate the prepared statement and close
         the cursor.
         """
-        if self._prepared:
-            try:
-                self._connection.cmd_stmt_close(self._prepared['statement_id'])
-            except errors.Error:
-                # We tried to deallocate, but it's OK when we fail.
-                pass
-            self._prepared = None
+        self.reset()
         super(MySQLCursorPrepared, self).close()
 
     def _row_to_python(self, rowdata, desc=None):
@@ -1111,17 +1134,22 @@ class MySQLCursorPrepared(MySQLCursor):
         """
         pass
 
-    def _handle_result(self, res):
+    def _handle_result(self, result):
         """Handle result after execution"""
-        if isinstance(res, dict):
+        if isinstance(result, dict):
             self._connection.unread_result = False
             self._have_result = False
-            self._handle_noresultset(res)
+            self._handle_noresultset(result)
         else:
-            self._description = res[1]
+            self._description = result[1]
             self._connection.unread_result = True
             self._have_result = True
 
+            if 'status_flag' in result[2]:
+                self._handle_server_status(result[2]['status_flag'])
+            elif 'server_status' in result[2]:
+                self._handle_server_status(result[2]['server_status'])
+
     def execute(self, operation, params=(), multi=False):  # multi is unused
         """Prepare and execute a MySQL Prepared Statement
 
@@ -1138,7 +1166,10 @@ class MySQLCursorPrepared(MySQLCursor):
             self._executed = operation
             try:
                 if not isinstance(operation, bytes):
-                    operation = operation.encode(self._connection.charset)
+                    charset = self._connection.charset
+                    if charset == 'utf8mb4':
+                        charset = 'utf8'
+                    operation = operation.encode(charset)
             except (UnicodeDecodeError, UnicodeEncodeError) as err:
                 raise errors.ProgrammingError(str(err))
 
@@ -1199,6 +1230,8 @@ class MySQLCursorPrepared(MySQLCursor):
 
         Returns a tuple or None.
         """
+        if self._cursor_exists:
+            self._connection.cmd_stmt_fetch(self._prepared['statement_id'])
         return self._fetch_row() or None
 
     def fetchmany(self, size=None):
@@ -1214,10 +1247,18 @@ class MySQLCursorPrepared(MySQLCursor):
     def fetchall(self):
         if not self._have_unread_result():
             raise errors.InterfaceError("No result set to fetch from.")
-        (rows, eof) = self._connection.get_rows(
-            binary=self._binary, columns=self.description)
+        rows = []
+        if self._nextrow[0]:
+            rows.append(self._nextrow[0])
+        while self._have_unread_result():
+            if self._cursor_exists:
+                self._connection.cmd_stmt_fetch(
+                    self._prepared['statement_id'], MAX_RESULTS)
+            (tmp, eof) = self._connection.get_rows(
+                binary=self._binary, columns=self.description)
+            rows.extend(tmp)
+            self._handle_eof(eof)
         self._rowcount = len(rows)
-        self._handle_eof(eof)
         return rows
 
 
@@ -1293,9 +1334,15 @@ class MySQLCursorNamedTuple(MySQLCursor):
 
         if row:
             # pylint: disable=W0201
-            self.named_tuple = namedtuple('Row', self.column_names)
+            columns = tuple(self.column_names)
+            try:
+                named_tuple = NAMED_TUPLE_CACHE[columns]
+            except KeyError:
+                named_tuple = namedtuple('Row', columns)
+                NAMED_TUPLE_CACHE[columns] = named_tuple
             # pylint: enable=W0201
-            return self.named_tuple(*row)
+            return named_tuple(*row)
+        return None
 
     def fetchone(self):
         """Returns next row of a query result set
@@ -1304,8 +1351,7 @@ class MySQLCursorNamedTuple(MySQLCursor):
         if row:
             if hasattr(self._connection, 'converter'):
                 return self._row_to_python(row, self.description)
-            else:
-                return row
+            return row
         return None
 
     def fetchall(self):
