@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -285,15 +285,27 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
             'status_flag': 1, 'affected_rows': 6}
         self.assertEqual(exp, self.cnx._handle_result(packet))
 
-        exp = [
-            bytearray(b'\x47\x00\x00\x04\x31\x09\x63\x31\x5f\x31\x09\x63\x32'
-                      b'\x5f\x31\x0a\x32\x09\x63\x31\x5f\x32\x09\x63\x32\x5f'
-                      b'\x32\x0a\x33\x09\x63\x31\x5f\x33\x09\x63\x32\x5f\x33'
-                      b'\x0a\x34\x09\x63\x31\x5f\x34\x09\x63\x32\x5f\x34\x0a'
-                      b'\x35\x09\x63\x31\x5f\x35\x09\x63\x32\x5f\x35\x0a\x36'
-                      b'\x09\x63\x31\x5f\x36\x09\x63\x32\x5f\x36'),
-            bytearray(b'\x00\x00\x00\x05')
-        ]
+        if os.name != 'nt':
+            exp = [
+                bytearray(b'\x47\x00\x00\x04\x31\x09\x63\x31\x5f\x31\x09\x63\x32'
+                          b'\x5f\x31\x0a\x32\x09\x63\x31\x5f\x32\x09\x63\x32\x5f'
+                          b'\x32\x0a\x33\x09\x63\x31\x5f\x33\x09\x63\x32\x5f\x33'
+                          b'\x0a\x34\x09\x63\x31\x5f\x34\x09\x63\x32\x5f\x34\x0a'
+                          b'\x35\x09\x63\x31\x5f\x35\x09\x63\x32\x5f\x35\x0a\x36'
+                          b'\x09\x63\x31\x5f\x36\x09\x63\x32\x5f\x36'),
+                bytearray(b'\x00\x00\x00\x05')
+            ]
+        else:
+            exp = [
+                bytearray(b'\x4c\x00\x00\x04\x31\x09\x63\x31\x5f\x31\x09\x63'
+                          b'\x32\x5f\x31\x0d\x0a\x32\x09\x63\x31\x5f\x32\x09'
+                          b'\x63\x32\x5f\x32\x0d\x0a\x33\x09\x63\x31\x5f\x33'
+                          b'\x09\x63\x32\x5f\x33\x0d\x0a\x34\x09\x63\x31\x5f'
+                          b'\x34\x09\x63\x32\x5f\x34\x0d\x0a\x35\x09\x63\x31'
+                          b'\x5f\x35\x09\x63\x32\x5f\x35\x0d\x0a\x36\x09\x63'
+                          b'\x31\x5f\x36\x09\x63\x32\x5f\x36'),
+                bytearray(b'\x00\x00\x00\x05')
+            ]
         self.assertEqual(exp, self.cnx._socket.sock._client_sends)
 
         # Column count is invalid ( more than 4096)
@@ -742,6 +754,7 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
     def test__do_auth(self):
         """Authenticate with the MySQL server"""
         self.cnx._socket.sock = tests.DummySocket()
+        self.cnx._handshake["auth_plugin"] = "mysql_native_password"
         flags = constants.ClientFlag.get_default()
         kwargs = {
             'username': 'ham',
@@ -781,6 +794,196 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         self.assertRaises(errors.ProgrammingError,
                           self.cnx._do_auth, **kwargs)
 
+    @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
+    @unittest.skipIf(tests.MYSQL_VERSION < (8, 0, 3),
+                     "caching_sha2_password plugin not supported by server.")
+    def test_caching_sha2_password(self):
+        """Authenticate with the MySQL server using caching_sha2_password"""
+        self.cnx._socket.sock = tests.DummySocket()
+        flags = constants.ClientFlag.get_default()
+        flags |= constants.ClientFlag.SSL
+        kwargs = {
+            'username': 'ham',
+            'password': 'spam',
+            'database': 'test',
+            'charset': 33,
+            'client_flags': flags,
+            'ssl_options': {
+                'ca': os.path.join(tests.SSL_DIR, 'tests_CA_cert.pem'),
+                'cert': os.path.join(tests.SSL_DIR, 'tests_client_cert.pem'),
+                'key': os.path.join(tests.SSL_DIR, 'tests_client_key.pem'),
+            },
+        }
+
+        self.cnx._handshake['auth_plugin'] = 'caching_sha2_password'
+        self.cnx._handshake['auth_data'] = b'h4i6oP!OLng9&PD@WrYH'
+        self.cnx._socket.switch_to_ssl = \
+            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
+
+        # Test perform_full_authentication
+        # Exchange:
+        # Client              Server
+        # ------              ------
+        # make_ssl_auth
+        # first_auth
+        #                     full_auth
+        # second_auth
+        #                     OK
+        self.cnx._socket.sock.reset()
+        self.cnx._socket.sock.add_packets([
+            bytearray(b'\x02\x00\x00\x03\x01\x04'), # full_auth request
+            bytearray(b'\x07\x00\x00\x05\x00\x00\x00\x02\x00\x00\x00') # OK
+        ])
+        self.cnx._do_auth(**kwargs)
+        packets = self.cnx._socket.sock._client_sends
+        self.assertEqual(3, len(packets))
+        ssl_pkt = self.cnx._protocol.make_auth_ssl(
+                charset=kwargs['charset'], client_flags=kwargs['client_flags'])
+        # Check the SSL request packet
+        self.assertEqual(packets[0][4:], ssl_pkt)
+        auth_pkt = self.cnx._protocol.make_auth(
+            self.cnx._handshake, kwargs['username'],
+            kwargs['password'], kwargs['database'],
+            charset=kwargs['charset'],
+            client_flags=kwargs['client_flags'],
+            ssl_enabled=True)
+        # Check the first_auth packet
+        self.assertEqual(packets[1][4:], auth_pkt)
+        # Check the second_auth packet
+        self.assertEqual(packets[2][4:],
+                bytearray(kwargs["password"].encode('utf-8') + b"\x00"))
+
+        # Test fast_auth_success
+        # Exchange:
+        # Client              Server
+        # ------              ------
+        # make_ssl_auth
+        # first_auth
+        #                     fast_auth
+        #                     OK
+        self.cnx._socket.sock.reset()
+        self.cnx._socket.sock.add_packets([
+            bytearray(b'\x02\x00\x00\x03\x01\x03'), # fast_auth success
+            bytearray(b'\x07\x00\x00\x05\x00\x00\x00\x02\x00\x00\x00') # OK
+        ])
+        self.cnx._do_auth(**kwargs)
+        packets = self.cnx._socket.sock._client_sends
+        self.assertEqual(2, len(packets))
+        ssl_pkt = self.cnx._protocol.make_auth_ssl(
+                charset=kwargs['charset'], client_flags=kwargs['client_flags'])
+        # Check the SSL request packet
+        self.assertEqual(packets[0][4:], ssl_pkt)
+        auth_pkt = self.cnx._protocol.make_auth(
+            self.cnx._handshake, kwargs['username'],
+            kwargs['password'], kwargs['database'],
+            charset=kwargs['charset'],
+            client_flags=kwargs['client_flags'],
+            ssl_enabled=True)
+        # Check the first auth packet
+        self.assertEqual(packets[1][4:], auth_pkt)
+
+    @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
+    def test_auth_plugin_is_not_supported_but_required(self):
+        """Test behavior when server's default auth_plugin is required"""
+        self.cnx._socket.sock = tests.DummySocket()
+        flags = constants.ClientFlag.get_default()
+        flags |= constants.ClientFlag.SSL
+        kwargs = {
+            'username': 'ham',
+            'password': 'spam',
+            'database': 'test',
+            'charset': 33,
+            'client_flags': flags,
+            'ssl_options': {
+                'ca': os.path.join(tests.SSL_DIR, 'tests_CA_cert.pem'),
+                'cert': os.path.join(tests.SSL_DIR, 'tests_client_cert.pem'),
+                'key': os.path.join(tests.SSL_DIR, 'tests_client_key.pem'),
+            },
+        }
+
+        self.cnx._handshake['auth_plugin'] = 'unsupported_auth_plugin'
+        self.cnx._handshake['auth_data'] = b'abcdef!012345'
+        self.cnx._socket.switch_to_ssl = \
+            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
+
+        # Test perform_full_authentication
+        # Exchange:
+        # Client              Server
+        # ------              ------
+        # make_ssl_auth
+        # first_auth
+        #                     Sorry user requires the unsupported_auth_plugin
+        # second_auth
+        #                     OK
+        self.cnx._socket.sock.reset()
+        self.cnx._socket.sock.add_packets([
+            bytearray(b'\x2c\x00\x00\x03\xfe\x75\x6e\x73\x75\x70\x70\x6f\x72'
+                      b'\x74\x65\x64\x5f\x61\x75\x74\x68\x5f\x70\x6c\x75\x67'
+                      b'\x69\x6e\x00\x60\x1e\x10\x78\x01\x3c\x1e\x33\x38\x6f'
+                      b'\x08\x5f\x0d\x7a\x6f\x01\x7b\x7a\x4a\x0d\x00')
+        ])
+        # Since the fictional authoritation plugin 'unsupported_auth_plugin' is
+        # not supported a NotSupportedError is raised
+        self.assertRaises(errors.NotSupportedError, self.cnx._do_auth, **kwargs)
+
+    @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
+    @unittest.skipIf(tests.MYSQL_VERSION < (8, 0, 3),
+                     "caching_sha2_password plugin not supported by server.")
+    def test_auth_plugin_fall_back_if_not_supported(self):
+        """Test Fall back to dflt auth_plugin if server's plugin is unknown"""
+        self.cnx._socket.sock = tests.DummySocket()
+        flags = constants.ClientFlag.get_default()
+        flags |= constants.ClientFlag.SSL
+        kwargs = {
+            'username': 'ham',
+            'password': 'spam',
+            'database': 'test',
+            'charset': 33,
+            'client_flags': flags,
+            'ssl_options': {
+                'ca': os.path.join(tests.SSL_DIR, 'tests_CA_cert.pem'),
+                'cert': os.path.join(tests.SSL_DIR, 'tests_client_cert.pem'),
+                'key': os.path.join(tests.SSL_DIR, 'tests_client_key.pem'),
+            },
+        }
+
+        self.cnx._handshake['auth_plugin'] = 'unsupported_auth_plugin'
+        self.cnx._handshake['auth_data'] = b'abcdef!012345'
+        self.cnx._socket.switch_to_ssl = \
+            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
+
+        # Test perform_full_authentication
+        # Exchange:
+        # Client              Server
+        # ------              ------
+        # make_ssl_auth
+        # first_auth
+        #                     auth_switch to mysql_native_password
+        # second_auth
+        #                     OK
+        self.cnx._socket.sock.reset()
+        self.cnx._socket.sock.add_packets([
+            bytearray(b'\x8d\xaa\x0b\x00\x00\x00'), # full_auth request
+            bytearray(b'\x00@!\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
+                      b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'),
+            bytearray(b'\x07\x00\x00\x05\x00\x00\x00\x02\x00\x00\x00') # OK
+        ])
+
+        # No exception should be raided
+        self.cnx._do_auth(**kwargs)
+
+        packets = self.cnx._socket.sock._client_sends
+
+        self.cnx._handshake['auth_plugin'] = 'mysql_native_password'
+        auth_pkt = self.cnx._protocol.make_auth(
+            self.cnx._handshake, kwargs['username'],
+            kwargs['password'], kwargs['database'],
+            charset=kwargs['charset'],
+            client_flags=kwargs['client_flags'],
+            ssl_enabled=True)
+        # Check the first_auth packet
+        self.assertEqual(packets[1][4:], auth_pkt)
+
     @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
     def test__do_auth_ssl(self):
         """Authenticate with the MySQL server using SSL"""
@@ -812,9 +1015,11 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
                 self.cnx._handshake, kwargs['username'],
                 kwargs['password'], kwargs['database'],
                 charset=kwargs['charset'],
-                client_flags=kwargs['client_flags']),
+                client_flags=kwargs['client_flags'],
+                ssl_enabled=True),
         ]
-        self.cnx._socket.switch_to_ssl = lambda ca, cert, key: None
+        self.cnx._socket.switch_to_ssl = \
+            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
         self.cnx._socket.sock.reset()
         self.cnx._socket.sock.add_packets([
             bytearray(b'\x07\x00\x00\x01\x00\x00\x00\x02\x00\x00\x00'),
@@ -1598,6 +1803,8 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
             else:
                 self.assertNotEqual((b'2',), self.cnx.get_rows()[0][0])
 
+    @unittest.skipIf(tests.MYSQL_VERSION <= (5, 7, 1), "Shutdown CMD "
+                     "not tested with MySQL version 5.6 (BugOra17422299)")
     def test_shutdown(self):
         """Shutting down a connection"""
         config = tests.get_mysql_config()
@@ -1671,10 +1878,16 @@ class WL7937(tests.MySQLConnectorTests):
         self.cur.execute(sql, (self.data_file, ))
         self.cur.execute("SELECT * FROM local_data")
 
-        exp = [
-            (1, 'c1_1', 'c2_1'), (2, 'c1_2', 'c2_2'),
-            (3, 'c1_3', 'c2_3'), (4, 'c1_4', 'c2_4'),
-            (5, 'c1_5', 'c2_5'), (6, 'c1_6', 'c2_6')]
+        if os.name != 'nt':
+            exp = [
+                (1, 'c1_1', 'c2_1'), (2, 'c1_2', 'c2_2'),
+                (3, 'c1_3', 'c2_3'), (4, 'c1_4', 'c2_4'),
+                (5, 'c1_5', 'c2_5'), (6, 'c1_6', 'c2_6')]
+        else:
+            exp = [
+                (1, 'c1_1', 'c2_1\r'), (2, 'c1_2', 'c2_2\r'),
+                (3, 'c1_3', 'c2_3\r'), (4, 'c1_4', 'c2_4\r'),
+                (5, 'c1_5', 'c2_5\r'), (6, 'c1_6', 'c2_6')]
         self.assertEqual(exp, self.cur.fetchall())
 
     def test_without_load_local_infile(self):
