@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -321,7 +321,7 @@ class MySQLCursorTests(tests.TestsCursor):
         self.cur = cursor.MySQLCursor(self.cnx)
         self.assertRaises(StopIteration, self.cur.__next__)
         self.cur.execute("SELECT BINARY 'ham'")
-        exp = (b'ham',)
+        exp = ('ham',)
         self.assertEqual(exp, next(self.cur))
         self.cur.close()
 
@@ -554,7 +554,7 @@ class MySQLCursorTests(tests.TestsCursor):
         self.assertTrue(tests.cmp_result(exp, self.cur._warnings))
 
         self.cur.execute("SELECT BINARY 'ham'")
-        exp = [(b'ham',)]
+        exp = [('ham',)]
         self.assertEqual(exp, self.cur.fetchall())
         self.cur.close()
 
@@ -826,7 +826,7 @@ class MySQLCursorTests(tests.TestsCursor):
         self.cnx = connection.MySQLConnection(**tests.get_mysql_config())
         self.cur = self.cnx.cursor()
         self.cur.execute("SELECT BINARY 'ham'")
-        exp = (b'ham',)
+        exp = ('ham',)
         self.assertEqual(exp, self.cur.fetchone())
         self.assertEqual(None, self.cur.fetchone())
         self.cur.close()
@@ -1113,7 +1113,7 @@ class MySQLCursorPreparedTests(tests.TestsCursor):
 
     def test_callproc(self):
         cur = self.cnx.cursor(cursor_class=cursor.MySQLCursorPrepared)
-        self.assertRaises(errors.NotSupportedError, cur.callproc)
+        self.assertRaises(errors.NotSupportedError, cur.callproc, None)
 
     def test_close(self):
         cur = self.cnx.cursor(cursor_class=cursor.MySQLCursorPrepared)
@@ -1232,7 +1232,7 @@ class MySQLCursorPreparedTests(tests.TestsCursor):
         self.assertEqual(3, cur.rowcount)
 
         cur.execute(stmt_select)
-        self.assertEqual([(1, b'100'), (2, b'200'), (3, b'300')],
+        self.assertEqual([(1, '100'), (2, '200'), (3, '300')],
                          cur.fetchall(), "Multi insert test failed")
 
         data = [(2,), (3,)]
@@ -1255,8 +1255,8 @@ class MySQLCursorPreparedTests(tests.TestsCursor):
             return row
         cur._fetch_row = _fetch_row
 
-        cur._test_fetch_row = [(b'ham',)]
-        self.assertEqual((b'ham',), cur.fetchone())
+        cur._test_fetch_row = [('ham',)]
+        self.assertEqual(('ham',), cur.fetchone())
         self.assertEqual(None, cur.fetchone())
 
     def test_fetchmany(self):
@@ -1340,6 +1340,7 @@ class MySQLCursorBufferedDictTests(tests.TestsCursor):
                          'name VARCHAR(20), city VARCHAR(20))')
 
     def tearDown(self):
+        self.cur.execute('DROP TABLE IF EXISTS MySQLCursorBufferedDictTests')
         self.cur.close()
         self.connection.close()
 
@@ -1414,6 +1415,9 @@ class MySQLCursorBufferedNamedTupleTests(tests.TestsCursor):
                          'id INT(10), name VARCHAR(20), city VARCHAR(20))')
 
     def tearDown(self):
+        self.cur.execute('DROP TABLE IF EXISTS '
+                         'MySQLCursorBufferedNamedTupleTests')
+
         self.cur.close()
         self.connection.close()
 
